// checkbook program version 5 for Fundamentals of Computing

import java.io.*;
import java.util.*;

public class chkbk {
    // variables declared here are global and
    // can be used by any of the routines within this class
    static double CurrentBalance = 0.0; // current account
    // set up an input stream so we can read lines
    // from the keyboard
    static BufferedReader kbd = new BufferedReader(new InputStreamReader(System.in));
    static PrintStream tty = new PrintStream(System.out);
    static Vector Xacts = new Vector();    // vector for transactions
    static String FileName = new String("checkbk.txt");


    public static void main(String args[])
        throws java.io.IOException
    {
        String buf; // input buffer
        int opt;    // menu option

        // initial message
        System.out.println("Welcome to the Java Checkbook Program");

        //
        // main loop
        //
        // keep displaying menu and getting options
        // until the user quits
        //
        do
        {
            System.out.println("Current balance is " + CurrentBalance);
            // main menu
            System.out.println("Options are:");
            System.out.println("1. Set Balance");
            System.out.println("2. Check");
            System.out.println("3. Deposit");
            System.out.println("4. MAC");
            System.out.println("5. Charge");
            System.out.println("6. Display History");
            System.out.println("7. Load from File");
            System.out.println("8. Save to File");
            System.out.println("9. Quit");

            // get menu option from user
            System.out.print("Enter Option: "); // prompt the user
            try
            {
            	buf = kbd.readLine();   // read what they type
            	opt = Integer.parseInt(buf);    // and convert to a number
            } catch (Exception e) { opt = 0; } // ignore option

            switch(opt)
            {
            case 1: set_balance(); break;
            case 2: process_check(); break;
            case 3: process_deposit(); break;
            case 4: process_mac(); break;
            case 5: process_chg(); break;
            case 6: display_history(); break;
            case 7: load_file(); break;
            case 8: save_file(); break;
            case 9: System.out.println("Bye!"); break;
            default:
                System.out.println("Invalid option - try again");
            }

        } while (opt != 9);
        System.out.println("Thanks for using our program!  Come again!");
    } // end main routine

    public static void set_balance()
    { // get new account balance from user
        String buf;
        Double amt;

        System.out.print("Enter new balance: ");
        try
        {
        	buf = kbd.readLine();
        } catch (Exception e) { return; }
        amt = new Double(buf);
        CurrentBalance = amt.doubleValue();
    }

    public static void process_check()
    {
        xact x = new xact("Chk");

        x.getXact(CurrentBalance);
        if (x != null)
        {
        	CurrentBalance = x.getBal();    // new balance
        	x.printXact(tty);
        	Xacts.addElement(x);    // add xact to vector
        }
    }

    public static void process_deposit()
    {
        xact x = new xact("Dep");

        x.getXact(CurrentBalance);
        if (x != null)
        {
	        CurrentBalance = x.getBal();    // new balance
    	    x.printXact(tty);
        	Xacts.addElement(x);    // add xact to vector
        }
    }

    public static void process_mac()
    {
        xact x;
        String buf;
        char ch;

        do
        {
            System.out.print("Transaction type (w, d, q): ");
            try
            {
            	buf = kbd.readLine();
            } catch (Exception e) { buf = ""; }
            if (buf.equalsIgnoreCase("w"))
            { // MAC withdrawl
                x = new xact("MACW");
            }
            else if (buf.equalsIgnoreCase("d"))
            { // MAC deposit
                x = new xact("MACD");
            }
            else
            { // invalid
                System.err.println("Invalid option (" + buf + ")");
                x = null;
            }

            if (x != null)
            {
            	x.getXact(CurrentBalance);
                CurrentBalance = x.getBal();
                x.printXact(tty);
        		Xacts.addElement(x);    // add xact to vector
            }
        } while (!buf.equals("q"));
    }

    public static void process_chg()
    {
        xact x = new xact("Chg");

        x.getXact(CurrentBalance);
        if (x != null)
        {
	        CurrentBalance = x.getBal();    // new balance
    	    x.printXact(tty);
        	Xacts.addElement(x);    // add xact to vector
        }
    }

    public static void display_history()
    { // display past transactions
        int i, m;	// loop variable & max elements
        xact x;     // current transaction

        m = Xacts.size();   // how many elements
        for (i=0; i < m; i++)
        {
            x = (xact)Xacts.elementAt(i);
            x.printXact(tty);
        }
    }

    public static void save_file()
    { // save current results to file
        String buf;
        PrintStream dat;
        int i, m;
        xact x;

        System.out.print("Enter file name (default " + FileName + "): ");
        try
        {
        	buf = kbd.readLine();   // get name of output file
            if (buf.length() > 0) FileName = buf;  // use default
            dat = new PrintStream(new FileOutputStream(FileName));
        } catch (Exception e)
        { // can't read file
            System.err.println("Can't open output file " + FileName);
            return;
        }

        // file open - write each transaction
        m = Xacts.size();

        for (i=0; i < m; i++)
        { // loop writing transactions
	        try
        	{ // catch I/O error
                x = (xact)Xacts.elementAt(i); // get transaction
                x.printXact(dat);           // print transaction
        	} catch (Exception e)
        	{ // I/O problem
            	System.err.println("I/O error while writing " + FileName);
            }
        }
    }

    static public void load_file()
    {
        String buf;
        BufferedReader dat;
        int i, m;
        xact x;
        boolean running = true;

        System.out.print("Enter input file name (default "
        		+ FileName + "): ");
        try
        {
            buf = kbd.readLine();
            if (buf.length() > 0) FileName = buf;
            dat = new BufferedReader(new InputStreamReader(new FileInputStream(FileName)));
        } catch (Exception e)
        { // can't open input file
            System.out.println("Can't open output file " + FileName);
            return;
        }

        // clear vector
        Xacts = new Vector(); // start new array

        while (running)
        { // loop reading lines until done
            try
            {
                buf = dat.readLine();
                if (buf == null) break; // EOF
            }
            catch(Exception e)
            { // I/O error
                break;
            }
            x = new xact(buf);
            x.printXact(tty);
            Xacts.addElement(x);
        }
        x = (xact)Xacts. lastElement();
        if (x != null) CurrentBalance = x.getBal();
        else CurrentBalance = 0.0;
    }
}

