// transaction class version 3 for checkbook program

import java.util.*;
import java.io.*;
import java.text.*;

class xact {
    String xtype;	// transaction type
    int xnum;    // check number
    Date xdate;     // date of transaction
    String xdesc;   // description of transaction
    double xamt;    // amount of transaction
    double xbal;    // balance after transaction

    public xact()
    {       // default constructor
        xtype = new String(); // transaction type
        xnum = 0;   // assume no check number
        xdate = new Date(); // default is today
        xdesc = new String(""); // no description
        xamt = 0.0;
        xbal = 0.0;
    }

    public xact(String str)
    { // parse string into fields
        String s;
        Double amt;
        StringTokenizer stok;
	  int cnt;

        // first, initialize to all empty fields
        xtype = new String(); // transaction type
        xnum = 0;   // assume no check number
        xdate = new Date(); // default is today
        xdesc = new String(""); // no description
        xamt = 0.0;
        xbal = 0.0;

        // now parse the line into fields
        stok = new StringTokenizer(str, "|");   // field delimeter
	  cnt = stok.countTokens();
        if (cnt != 6 && cnt != 1)
        {
        	System.err.println("Invalid input line (" + str + "), cnt=" + cnt);
            return; //
        }

        xtype = stok.nextToken();   // transaction type
	  if (cnt > 1)
	  { // entire line, not just type
          s = stok.nextToken();       // check number
          xnum = Integer.parseInt(s.trim());
      	  DateFormat df = DateFormat.getDateInstance();
      	  try
      	  {
              	setDate(df.parse(stok.nextToken()));
      	  } catch (Exception e)
      	  { // bad date
              	setDate(new Date());	// use today's date instead
      	  }
            xdesc = stok.nextToken();   // description of transaction
            s = stok.nextToken();       // get amount as string
            amt = new Double(s);
            xamt = amt.doubleValue();
            s = stok.nextToken();       // get balance as string
            amt = new Double(s);
	      xbal = amt.doubleValue();
	  }
	  else
	  { // just type - fill in rest of objects
          xnum = 0;   // assume no check number
          xdate = new Date(); // default is today
          xdesc = new String(""); // no description
          xamt = 0.0;
          xbal = 0.0;
	  }
    }

    public xact(String xt, int xn, Date xda, String xde,
                double xa, double xb)
    { // normal constructor
        xtype = xt;
        xnum = xn;
        xdate = xda;
        xdesc = xde;
        xamt = xa;
        xbal = xb;
    }

    public void printXact(PrintStream dat)
    {
	  DateFormat df = DateFormat.getDateInstance();
	  dat.println(xtype + " | " + xnum + " | " + df.format(xdate)
					+ " | " + xdesc + " | " + xamt + " | " + xbal);
    }

    public void setType(String xt) { xtype = xt; }
    public void setNum(int n) { xnum = n; }
    public void setDate(Date d) { xdate = d; }
    public void setDesc(String s) { xdesc = s; }
    public void setAmt(double a) { xamt = a; }
    public void setBal(double b) { xbal = b; }

    public String getType() { return xtype; }
    public int getNum() { return xnum; }
    public Date getDate() { return xdate; }
    public String getDesc() { return xdesc; }
    public double getAmt() { return xamt; }
    public double getBal() { return xbal; }

    public boolean isCheck() { return xtype.equals("Chk"); }

    public int getSign()
    { // returns +1 if amount added to balance, -1 if subtracted
        if (xtype.equals("Chk")) return -1;
        if (xtype.equals("Dep")) return +1;
        if (xtype.equals("MACD")) return +1;
        if (xtype.equals("MACW")) return -1;
        if (xtype.equals("Chg")) return -1;
        System.out.println("Invalid transaction type");
        return 0;   // error return
    }

    public void getXact(double oldbal)
    {
    	  BufferedReader kbd = new BufferedReader(new InputStreamReader(System.in));
        String buf;
        Double amt;

        if (isCheck())
        { // this is a check - get the check number
            System.out.print("Enter Check Number: ");
            try {
                buf = kbd.readLine();
            } catch (Exception e) { return; };
            setNum(Integer.parseInt(buf));
        }

        System.out.print("Enter Date (mmm dd, yyyy): ");
        try {
        	buf = kbd.readLine();
		DateFormat df = DateFormat.getDateInstance();
        	setDate(df.parse(buf));
        } catch (Exception e) 
        {
            System.err.println("Invalid date - using today's date");
        	setDate(new Date());
        }

        System.out.print("Enter Description: ");
        try {
        	buf = kbd.readLine();
        } catch (Exception e) { return; }
        setDesc(buf);

        System.out.print("Enter Amount: ");
        try {
        	buf = kbd.readLine();
        } catch (Exception e) { return; }
	  try {
        	amt = new Double(buf);
	  } catch (Exception e) 
	  { 
		amt = new Double(0.0); 
		System.out.println("Invalid amount (" + buf + "). Using zero");
	  }	// invalid number
        setAmt(amt.doubleValue());

        // new compute balance
        switch (getSign())
        {
            case 1: // add to old balance
                xbal = oldbal + xamt;
                break;
            case -1: // subtract from old balance
                xbal = oldbal - xamt;
                break;
            default:
                xbal = oldbal;   // no change
                System.out.println("Invalid Transaction type");
                break;
        }

    }
}

